# Useful information:
#
# https://en.wikipedia.org/wiki/List_of_physical_quantities
# https://en.wikipedia.org/wiki/List_of_physical_constants
# https://www.themeasureofthings.com/
# https://cs.uwaterloo.ca/~dtompkin/music/bpm/118.html
# https://physics.stackexchange.com/questions/167720/what-is-the-correct-theoretical-formula-for-the-hyperfine-splitting-of-neutral-h
#
# "free" Unitful units:  f jno r v xzDEIOQUXZ |
# "used" Unitful units: e i   p t w           |
module IIPAU

include("Seximal.jl")
include("Quantum.jl")
include("Natural.jl")
using .Seximal
using .Quantum
using .Natural

import Unitful
using Unitful: @unit, @affineunit, @u_str
export item
export @b6_str
export yocte, zepte, atte, femte, pice, nane, micre, mille, cente, dice
export yotte, zette, exe, pete, tere, gige, mege, kile, hecte, dece
export to₆

export wave_number

export span, tick, egg, spark, lotta, alotta, brr
export ball, touch, jolt, cell, wire

export wg

function __init__()
    Unitful.register(IIPAU)
end

# Base Units
span = 1@unit ṡ "ṡ" Span (Unitful.c / Natural.HHz) false # ~21cm
tick = 1@unit t "t" Tick (b6"2e15" / Natural.HHz) false # ~0.51s (1/tick is ~118 bpm)
egg = 1@unit e "e" Egg (-b6"4e44" * Natural.ΔEₕ * tick^2 / span^2) false # ~136g
spark = 1@unit ş "ş" Spark (b6"1e34" * Unitful.q) false # ~0.02C
lotta = 1@unit ļ "ļ" Lotta (1u"mol" * egg / u"g") false # ~135mol
alotta = (lotta |> item).val # ~81 septillion (81e24)
brr = 1@unit bʳ "bʳ" Brr (Natural.ΔEₕ / Natural.hexit) false # ~0.04K
@affineunit °c "°c" (Int(b6"-100000") * brr) # ~23°C -> 0°c, human's can't really survive past about -2000₆°c (too hot)

# Derived Units
ball = 1@unit ḃ "ḃ" Ball ((π/6)*span^3) false # ~5L (sphere of diameter 1ṡ)
touch = 1@unit ṫ "ṫ" Touch (egg * span / tick^2) false # ~0.1N (10grams of weight)
jolt = 1@unit j "j" Jolt (spark / tick) false # ~0.04A
cell = 1@unit ċ "ċ" Cell (span * touch / spark) false # ~1.1V (just shy of zinc/copper electrode)
wire = 1@unit w "w" Wire (span^2 * egg / spark^2 / tick) false # ~26.6Ω almost exactly 1 kile span of b6"2e-3"-span diameter copper wire (especially if the copper has some impurities)

# Convenience Units
instant = 1@unit i "i" Instant (b6"3e-2" * tick) false # ~5ms (300₆ instants to a tick)
pause = 1@unit ṗ "ṗ" Pause (b6"300" * tick) false # ~55 seconds
wait = 1@unit ẇ "ẇ" Wait (b6"300" * pause) false # ~100 minutes

# Useful references
resistance_of_wire(ρ, L, d) = ρ * L / (π * (d/2)^2)
ρAg = 1.59e-8u"Ω*m"
ρCu = 1.68e-8u"Ω*m"

gauge = 1@unit WG "WG" WireGauge (b6"1e-4" * span) false
wg(n) = 3 * span * 6.0^(-n/6) # wg(24) is the gauge for the wire unit
wgo(n) = resistance_of_wire(ρCu, kile * span, wg(n))
ρIdealWire = wire * π * (wg(24) / 2)^2 / (kile * span) |> u"m * Ω"

# temporary definition of american wire gauge:
awg(n) = 0.127u"mm" * 92^((36-n)/39)


#egg = 1@unit e "e" Egg (b6"1e53" * (Unitful.mp + Unitful.me)) false # ~80g

#=
# pace = 1@unit p "p" Pace (lₛ * b6"1e114") false              # ~86cm
# tick = 1@unit t "t" Tick (tₛ * b6"3e132") false              # ~0.52s = 300₆ instants (1/tick ~115bpm)
# egg = 1@unit e "e" Egg (mₛ * b6"1e14") false                 # ~112g
# jolt = 1@unit j "j" Jolt (b6"1e34" * Unitful.q / tick) false # ~40mA painful jolt (https://electronics.stackexchange.com/questions/19103/how-much-voltage-current-is-dangerous)
# brr = 1@unit ḃ "ḃ" Brr (Tₚ * b6"1e-110") false               # ~0.3K
# @affineunit °c "°c" (Int(b6"4400") * brr)                    # ~13°C -> 0°c

# Convenience Units
instant = 1@unit i "i" Instant (tₛ * b6"1e130") false # ~5ms (1000₆ instants is about a second)
pause = 1@unit ṗ "ṗ" Pause (tick * b6"300") false # ~1 minute
wait = 1@unit ẇ "ẇ" Wait (pause * b6"100") false # ~34 minutes
# TODO: name ~0.5L which is a mille pace^3?

# Derived Units
walk = 1@unit w "w" Walk (1 * pace / tick) false # ~6km/h (pretty quick walk)
# touch = 1@unit ṫ "ṫ" Touch (1 * egg * pace / tick ^ 2) false # ~0.36N (half what it takes to press a key on a keyboard)
nosh = 1@unit n "n" Nosh (b6"1e11" * pace * touch) false # ~20.5 kcal

#= Useful Constants ============================================================
 Earth's gravitational constant: gₑ ≈ 3₆p/t²
 Speed of light: c ≡ 3e14₆p/t
===============================================================================#

=#

#=
export stride, skip, bed, gulp
export moment, instant, wait
export amble, shot
export ball
export brr
export press, bite, snack
export jolt
export lotta, alotta

# Lengths (root unit: skip)
skip = 1@unit sk "sk" Skip (lₚ * b6"1e112") false               # ~28cm
stride = 1@unit st "st" Stride (dece * skip) false              # ~1.7m
sprint = 1@unit sp "sp" Sprint (kile * skip) false              # ~360m
# Note: a kile stride is about 1 nautical mile

# Areas (root unit: paper)
paper = 1@unit pa "pa" Paper (skip * skip) false                # ~0.08m²
bed = 1@unit bd "bd" Bed (stride * stride) false                # ~2.8m²

# Volumes (root unit: gulp)
gulp = 1@unit gu "gu" Gulp ((dice * IIPAU.skip) ^ 3) false      # ~100 mL

# Durations (root unit: instant)
instant = 1@unit in "in" Instant (tₚ * b6"1e131") false          # ~1/3s
moment = 1@unit mo "mo" Moment (dece * instant)  false           # ~2s
wait = 1@unit w "w" Wait (kile * moment) false                   # ~43.9 minutes

# Speed (root unit: amble)
amble = 1@unit amb "amb" Amble (skip / instant) false           # ~3km/h
shot = 1@unit sh "sh" Shot (kile * amble) false                 # ~1km/s (bullet speed)

# Mass (root unit: ball)
ball = 1@unit bl "bl" Ball (mₚ * b6"1.0e13") false              # ~200g (note: Float to fix Int64 overflow)

# Temperate (root unit: brr, base unit degree of comfort)
brr = 1@unit br "br" Brr (Tₚ * b6"1e-110") false                # ~0.3K
@affineunit °c "°c" (Int(b6"4400") * brr)                       # ~13°C -> 0°c
# This sets up that 0 is uncomfortably cold, 100₆ is comfortable, and 200₆ is
# uncomfortably warm (13, 23.5, 34)°C respectively

# Force (root unit: press)
press = 1@unit pr "pr" Press (ball * skip / instant^2) false    # ~0.5N (key press on keyboard)

# NOTE: 1bi / Mto = 1st, aka a bite is one megatouch stride (or a kileshove stride)
# NOTE: 1sn / Mpr = 1sk, aka a snack is one megepress skip

# Energy (root unit: snack)
snack = 1@unit sn "sn" Snack (b6"1.0e12" * press * skip) false                    # ~60cal
bite = 1@unit bi "bi" Bite (dice * snack) false # ~10cal

# Electricity, based on a common static shock
# ~0-2mJ / ~1-5KV = ~5e-8 to 5e-7C

# Charge (root unit: jolt)
jolt = 1@unit jo "jo" Jolt (b6"1e23" * Unitful.q) false         # ~7.5e-8C (a moderate static shock of charge)

# TODO: Voltage and consider charge in lotta electron charges (q in Unitful
# [leads to about 2e7C, which is a lot, but might make cleaner 'volts'])
# or consider using resistivity or conductivity of something common?
temp = 1@unit tmp "tmp" Temp (jolt / moment) false

# "Count" (root unit: lotta [mole equivalent]) ~1.32e26
lotta = 1@unit lo "lo" Lotta (1u"mol" * 1u"bl" / 1u"g") false   # ~220mol
alotta = (1u"lo" |> item).val                                   # ~1.32e26

function prefer_iipau()
    Unitful.preferunits(u"sk,mo,bl,br,lo,tmp"...)
end

function prefer_natural()
    Unitful.preferunits(u"lₚ,mₚ,tₚ,Tₚ,qₚ,Qₚ,count"...)
end
=#

end # module IIPAU
